

using namespace std; 

#include <cstdio>
#include <cfloat>
#include <chrono>
#include <ctime>
#include <unistd.h>

#define NANO_PER_SEC ((__clock_t) 1000000000)
#define NANO_PER_MILLI  ((__clock_t) 1000000)
#define MILLI_PER_SEC  ((__clock_t) 1000)

timespec diff(timespec start, timespec end)
{
    timespec temp;
    if ((end.tv_nsec - start.tv_nsec) < 0) {
        temp.tv_sec = end.tv_sec - start.tv_sec - 1;
        temp.tv_nsec = NANO_PER_SEC + end.tv_nsec - start.tv_nsec;
    }
    else {
        temp.tv_sec = end.tv_sec - start.tv_sec;
        temp.tv_nsec = end.tv_nsec - start.tv_nsec;
    }
    return temp;
}

clock_t gettotalnanosec(const timespec& time)
{
    return time.tv_sec * NANO_PER_SEC + time.tv_nsec;
}

clock_t gettotalmillisec(const timespec& time)
{
    return time.tv_sec * MILLI_PER_SEC + time.tv_nsec / NANO_PER_MILLI;
}

// 
// strace ./ConsoleApplication1.out
int main()
{
    {
        clock_t dwStart = (1000.0 * std::clock()) / CLOCKS_PER_SEC;
        usleep(1000 * 1000);
        clock_t dwEnd = (1000.0 * std::clock()) / CLOCKS_PER_SEC;

        clock_t dwDiff = dwEnd - dwStart;
        printf("%d, %d\n", dwStart, dwDiff);
    }

    {
        clock_t dwStart = std::clock();

        int sum = 0;
        for (int j = 0; j < 100; j++)
        for (int i = 0; i < 1000000; i++)
        {
            sum += i;
        }

        clock_t dwEnd = std::clock();

        clock_t dwDiff = dwEnd - dwStart;
        printf("%d, %d, %f\n", dwStart, dwDiff, 1000.0 * dwDiff / CLOCKS_PER_SEC);
    }

    struct timespec s_tspec;
    struct timespec e_tspec;

    {
        clock_gettime(CLOCK_MONOTONIC, &s_tspec);
        usleep(1500 * 1000);
        clock_gettime(CLOCK_MONOTONIC, &e_tspec);

        timespec diffspec = diff(s_tspec, e_tspec);
        clock_t timeDiff = gettotalmillisec(diffspec);

        printf("%lu\n", timeDiff);
    }

    {
        clock_gettime(CLOCK_MONOTONIC, &s_tspec);
        usleep(500 * 1000);
        clock_gettime(CLOCK_MONOTONIC, &e_tspec);

        timespec diffspec = diff(s_tspec, e_tspec);
        clock_t timeDiff = gettotalmillisec(diffspec);

        printf("%lu\n", timeDiff);
    }

    {
        clock_gettime(CLOCK_PROCESS_CPUTIME_ID, &s_tspec);
        usleep(1000 * 1000);

        int sum = 0;
        for (int j = 0; j < 170; j++)
            for (int i = 0; i < 1000000; i++)
            {
                sum += i;
            }

        clock_gettime(CLOCK_PROCESS_CPUTIME_ID, &e_tspec);

        timespec diffspec = diff(s_tspec, e_tspec);
        clock_t timeDiff = gettotalmillisec(diffspec);
        printf("%lu\n", timeDiff);
    }

    return 0;
}


//
//strace ./ConsoleApplication1.out
//execve("./ConsoleApplication1.out", ["./ConsoleApplication1.out"], [/* 18 vars */]) = 0
//brk(NULL) = 0x12e7000
//...[생략]...
//clock_gettime(CLOCK_PROCESS_CPUTIME_ID, { 0, 0 }) = 0
//nanosleep({ 1, 0 }, NULL) = 0
//clock_gettime(CLOCK_PROCESS_CPUTIME_ID, { 0, 265625000 }) = 0
//write(1, "0, 265\n", 70, 265
//) = 7
//clock_gettime(CLOCK_PROCESS_CPUTIME_ID, { 0, 265625000 }) = 0
//nanosleep({ 1, 0 }, NULL) = 0
//clock_gettime(CLOCK_PROCESS_CPUTIME_ID, { 0, 546875000 }) = 0
//write(1, "265625, 281250\n", 15265625, 281250
//) = 15
//exit_group(0) = ?
//++ + exited with 0 ++ +
//
